pub enum MoveCursor {
Before,
After,
}Expand description
MoveCursor determines how the cursor will resolve its position if the item originally referenced by the cursor is removed.
With MoveCursor::Before, the cursor will shift to the previous item that was visible at the time of cursor creation..
If no previous item is found that’s still visible, the cursor will dereference to 0.
With MoveCursor::After, the cursor will shift to the next item that was visible at the time of cursor creation.
If no next item is found that’s still visible, the cursor will dereference to sequence.length.
Variants