pub enum PatchAction {
PutMap {
key: String,
value: (Value<'static>, ObjId),
conflict: bool,
},
PutSeq {
index: usize,
value: (Value<'static>, ObjId),
conflict: bool,
},
Insert {
index: usize,
values: SequenceTree<(Value<'static>, ObjId, bool)>,
},
SpliceText {
index: usize,
value: ConcreteTextValue,
marks: Option<MarkSet>,
},
Increment {
prop: Prop,
value: i64,
},
Conflict {
prop: Prop,
},
DeleteMap {
key: String,
},
DeleteSeq {
index: usize,
length: usize,
},
Mark {
marks: Vec<Mark>,
},
}Variants
PutMap
A key was created or updated in a map
Fields
value: (Value<'static>, ObjId)The value that was inserted and the object ID of the object that was inserted. Note
that the Object ID is only meaningful for Value::Obj values
conflict: boolWhether there is a conflict at this key. If there is a conflict this patch represents
the “winning” value of the conflict. The conflicting values can be obtained with
crate::ReadDoc::get_all
PutSeq
An index in a sequence was updated
Fields
value: (Value<'static>, ObjId)The value that was set and the object ID of the object that was set. Note that the
Object ID is only meaningful for Value::Obj values
conflict: boolWhether there is a conflict at this index. If there is a conflict this patch represents
the “winning” value of the conflict. The conflicting values can be obtained with
crate::ReadDoc::get_all
Insert
One or more elements were inserted into a sequence
Fields
values: SequenceTree<(Value<'static>, ObjId, bool)>The values that were inserted, in order that they appear. As with Self::PutMap and
Self::PutSeq the object ID is only meaningful for Value::Obj values
SpliceText
Some text was spliced into a text object
Fields
value: ConcreteTextValueThe text that was inserted
Increment
A counter was incremented
Fields
Conflict
A new conflict has appeared
DeleteMap
A key was deleted from a map
DeleteSeq
One or more indices were removed from a sequence
Mark
Some marks within a text object were added or removed