pub struct ExpandedChange {
pub operations: Vec<Op>,
pub actor_id: ActorId,
pub hash: Option<ChangeHash>,
pub seq: u64,
pub start_op: NonZeroU64,
pub time: i64,
pub message: Option<String>,
pub deps: Vec<ChangeHash>,
pub extra_bytes: Vec<u8>,
}Expand description
A change represents a group of operations performed by an actor.
Fields
operations: Vec<Op>The operations performed in this change.
actor_id: ActorIdThe actor that performed this change.
hash: Option<ChangeHash>The hash of this change.
seq: u64The index of this change in the changes from this actor.
start_op: NonZeroU64The start operation index. Starts at 1.
time: i64The time that this change was committed.
message: Option<String>The message of this change.
deps: Vec<ChangeHash>The dependencies of this change.
extra_bytes: Vec<u8>Trait Implementations
Sourceimpl<'de> Deserialize<'de> for Change
impl<'de> Deserialize<'de> for Change
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl From<&Change> for ExpandedChange
impl From<&Change> for ExpandedChange
Sourceimpl From<Change> for Change
impl From<Change> for Change
Sourcefn from(e: ExpandedChange) -> Self
fn from(e: ExpandedChange) -> Self
Converts to this type from the input type.