Skip to main contentModule transaction
Source - CommitOptions
- Optional metadata for a commit.
- Failure
- The result of a failed, and rolled back, transaction.
- OwnedTransaction
- A transaction that owns the
Automerge document. - Success
- The result of a successful, and committed, transaction.
- Transaction
- A transaction on a document.
Transactions group operations into a single change so that no other operations can happen
in-between.
- BlockOrText
- Transactable
- A way of mutating a document within a single change.
- Result